/*
 *  TOPPERS/SSP Kernel
 *      Smallest Set Profile Kernel
 *
 *  Copyright (C) 2013-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2018,2019 by Naoki Saito
 *             Nagoya Municipal Industrial Research Institute, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: core_timer.h 384 2019-04-16 11:01:09Z nmir-saito $
 */

/*
 *  タイマドライバ（Generic Timer）
 */

#ifndef TOPPERS_CORE_TIMER_H
#define TOPPERS_CORE_TIMER_H

#include <sil.h>

/*
 *  タイマ割込みハンドラ登録のための定数
 */

#define INTNO_TIMER     TINTNO_CNTPNS  /* 割込み番号 */
#define INHNO_TIMER     INTNO_TIMER   /* 割込みハンドラ番号 */

#ifndef INTPRI_TIMER
#define INTPRI_TIMER    (-3)            /* 割込み優先度 */
#endif

#ifndef INTATR_TIMER
#define INTATR_TIMER    TA_NULL            /* 割込み属性 */
#endif

/*
 *  制御レジスタのビット定義
 */
#define CNTP_CTL_ENABLE     (1 << 0)
#define CNTP_CTL_MASK       (1 << 1)
#define CNTP_CTL_ISTATUS    (1 << 2)


#ifndef TOPPERS_MACRO_ONLY

/*
 *  物理タイマの制御レジスタからの読み出し
 */
Inline uint32_t
clock_read_ctrl(void) {
    uint32_t cntv_ctrl;

    Asm ("mrs %0, cntp_ctl_el0"
        : "=r"(cntv_ctrl) :: "memory");

    return cntv_ctrl;
}

/*
 *  物理タイマの制御レジスタへの書き込み
 */
Inline void
clock_write_ctrl(uint32_t cntv_ctrl) {

    Asm ("msr cntp_ctl_el0, %0"
        :: "r"(cntv_ctrl) : "memory");
}

/*
 *  物理タイマ(フリーカウンタ)のカウント値の読み出し
 */
Inline uint64_t
clock_read_counter(void){
    uint64_t cval;

    Asm ("mrs %0, cntpct_el0" : "=r" (cval):: "memory");

    return cval;
}

/*
 *  物理タイマ値の読み出し
 */
Inline uint32_t
clock_read_tval(void){
    uint32_t cntp_tval;

    Asm ("mrs %0, cntp_tval_el0; isb;" : "=r"(cntp_tval)
        :: "memory");
    return cntp_tval;
}

/*
 *  物理タイマ値の書き込み
 */
Inline void
clock_write_tval(uint32_t cntv_tval){

    Asm ("msr cntp_tval_el0, %0" :: "r"(cntv_tval): "memory");
}

/*
 *  タイマの比較値の読み出し
 */
Inline uint64_t
clock_read_cval(void){
    uint64_t cntp_cval;

    Asm ("mrs %0, cntp_cval_el0" : "=r"(cntp_cval)
        :: "memory");
    return cntp_cval;
}

/*
 *  タイマの比較値の書き込み
 */
Inline void
clock_write_cval(uint64_t cntp_cval){

    Asm ("msr cntp_cval_el0, %0" :: "r"(cntp_cval): "memory");
}

/*
 *  システムカウンタのクロック周波数の取得
 */
Inline uint32_t
get_timer_freq(void) {
    uint32_t freq_hz;

    Asm ("mrs %0, cntfrq_el0" : "=r"(freq_hz) :: "memory");
    return freq_hz;
}

/*
 *  タイマの起動処理
 *
 *  タイマを初期化し，周期的なタイマ割込み要求を発生させる．
 */
extern void target_timer_initialize(intptr_t exinf);

/*
 *  タイマの停止処理
 */
extern void target_timer_terminate(intptr_t exinf);

/*
 *  タイマ割込みハンドラ
 */
extern void target_timer_handler(void);

#endif /* TOPPERS_MACRO_ONLY */
#endif /* TOPPERS_PRC_TIMER_H */
